import tempfile
import unittest

from .. import api, types


class ApiTest(unittest.TestCase):
    def test_probabilities_no_valid_license(self) -> None:
        p = api.probabilities("foo")
        self.assertEqual(len(types.KNOWN_LICENSES), len(p))
        self.assertEqual(0.0, sum(i[1] for i in p))

    def test_guess_no_valid_license(self) -> None:
        g = api.guess_text("foo")
        self.assertEqual(None, g)

    def test_guess_filename(self) -> None:
        g = api.guess_file("LICENSE")
        assert g is not None
        self.assertEqual("MIT", g.shortname)

    def test_guess_filename_latin1(self) -> None:
        with tempfile.NamedTemporaryFile() as f:
            with open("LICENSE", "rb") as f2:
                data = f2.read()
            f.write(data + b"\xe9")
            f.flush()
            g = api.guess_file(f.name)
            assert g is not None
            self.assertEqual("MIT", g.shortname)

    def test_bsd_exact(self) -> None:
        bsd = types.license_by_shortname("BSD-2-Clause")
        g = api.guess_text(bsd.text)
        self.assertEqual(bsd, g)

    def test_bsd_some_words_changed(self) -> None:
        bsd = types.license_by_shortname("BSD-2-Clause")
        text = bsd.text
        text = text.replace("<year>", "1900").replace("<owner>", "Amusing Company Name")
        self.assertIn("Amusing Company Name", text)
        g = api.guess_text(text)
        self.assertEqual(bsd, g)

    def test_license_by_shortname_invalid(self) -> None:
        with self.assertRaises(KeyError, msg="Unknown license: 'Foo'"):
            types.license_by_shortname("Foo")



import unittest
from unittest import mock

from .. import cmdline


class CmdlineTest(unittest.TestCase):
    def test_cmdline(self) -> None:
        with mock.patch("infer_license.cmdline.print") as print_mock:
            cmdline.main(["LICENSE"])
            print_mock.assert_called_with("LICENSE: MIT")

    def test_cmdline_error_handling(self) -> None:
        with mock.patch("infer_license.cmdline.print") as print_mock:
            cmdline.main(["does_not_exist"])
            print_mock.assert_called_with("does_not_exist: Unknown")

    def test_cmdline_no_args(self) -> None:
        with mock.patch("infer_license.cmdline.print") as print_mock, mock.patch(
            "infer_license.cmdline.sys.argv", ["cmd", "LICENSE"]
        ):
            cmdline.main()
            print_mock.assert_called_with("LICENSE: MIT")



from setuptools import setup

setup(
    use_scm_version=True,
    install_requires=["dataclasses >= 0.7; python_version < '3.7'",],
    entry_points={"console_scripts": ["infer_license = infer_license.cmdline:main"]},
    package_data={"infer_license": ["py.typed"]},
)



import sys
from typing import List, Optional

from .api import guess_file


# TODO use click
def main(filenames: Optional[List[str]] = None) -> None:
    if filenames is None:
        filenames = sys.argv[1:]

    for filename in filenames:
        try:
            result = guess_file(filename)
        except IOError:
            result = None
        print(f"{filename}: {result.shortname if result else 'Unknown'}")


if __name__ == "__main__":  # pragma: no cover
    main()
